#include <algorithm>

#include "SurfaceQuadrature.h"
#include "Quadrature.h"

namespace AMDiS {

  //std::list<SurfaceQuadrature*> SurfaceQuadrature::surfaceQuadratureList;

  SurfaceQuadrature::SurfaceQuadrature(Quadrature *quad,
				       VectorOfFixVecs<DimVec<double> > &coords)
    : Quadrature((quad->getName() + " surface").c_str(),
		 quad->getDegree(),
		 quad->getDim() + 1,
		 quad->getNumPoints(),
		 NULL,
		 quad->getWeight()), 
      quad_(quad),
      coords_(coords)
  {
    int i, j, k;

    // copy coords
    // coords_ = new DimVec<double>[dim](dim, NO_INIT);
    //   for(i = 0; i < dim; i++) {
    //     coords_[i] = coords[i];
    //   }
  
    lambda = new VectorOfFixVecs<DimVec<double> >(dim, n_points, NO_INIT);

    // for each integration point
    for(i=0; i < n_points; i++) {
      // get coords of quadrature point in dim-1
      DimVec<double> origin = quad->getLambda(i);

      for(j=0; j < dim+1; j++)
	(*lambda)[i][j] = 0.0;

      for(j = 0; j < dim; j++) {
	for(k = 0; k < dim+1; k++) {
	  (*lambda)[i][k] += origin[j] * coords_[j][k];
	}
      }
    
      //     // create barycentric coords for dim
      //     if(dim > 1) {
      //       for (j = 0; j < side; j++)
      // 	(*lambda)[i][j] = origin[j];

      //       (*lambda)[i][side] = 0.0;
      
      //       for (j = side+1; j <= dim; j++)
      // 	(*lambda)[i][j] = origin[j-1];
      //     } else {
      //       (*lambda)[i][side] = 1.0;
      //       (*lambda)[i][1-side] = 0.0;
      //     }
    }
  }

  void
  SurfaceQuadrature::scaleSurfaceQuadrature(VectorOfFixVecs<DimVec<double> >&coords)
  {
    int i, j, k;

    // copy coords
    for(i = 0; i < dim; i++) {
      coords_[i] = coords[i];
    }
  
    // for each integration point
    for(i=0; i < n_points; i++) {
      // get coords of quadrature point in dim-1
      DimVec<double> origin = quad_->getLambda(i);

      for(j=0; j < dim+1; j++)
	(*lambda)[i][j] = 0.0;

      for(j = 0; j < dim; j++) {
	for(k = 0; k < dim+1; k++) {
	  (*lambda)[i][k] += origin[j] * coords_[j][k];
	}
      }
    }
  }

}
